<?php
/**
 * @package JAMP::plugins::system
 * @subpackage jamp
 * @author Joomla! Extensions Store
 * @copyright (C)2015 Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined ( '_JEXEC' ) or die ();

jimport ( 'joomla.plugin.plugin' );

/**
 * JAmp plugin for Joomla
 *
 * @package JAMP::plugins::system
 * @subpackage jamp
 * @since 1.0
 */
class PlgSystemJAmp extends JPlugin {
	/**
	 * Original JUri object including the AMP path
	 *
	 * @access private
	 * @var Object
	 */
	private $originalUri;
	
	/**
	 * @access private
	 * @var boolean
	 */
	private $debugMode;
	
	/**
	 * Store if the app can be executed after successfully checks
	 *
	 * @access protected
	 * @var int
	 */
	protected $startApp = false;
	
	/**
	 * App reference
	 *
	 * @access protected
	 * @var Object
	 */
	protected $app;
	
	/**
	 * Document reference
	 *
	 * @access protected
	 * @var Object
	 */
	protected $doc;
	
	/**
	 * Check if the Joomla multilanguage is enabled
	 * 
	 * @access private
	 * @return bool
	 */
	private function isMultilangEnabled() {
		// Status of language filter plugin.
		static $enabled = null;
	
		// If being called from the front-end, we can avoid the database query.
		if (is_null($enabled) && $this->app->isSite()) {
			$enabled = $this->app->getLanguageFilter();
			return $enabled;
		}
	
		return $enabled;
	}
	
	/**
	 * Check for the valid filtered execution of the plugin based on option and view selected
	 * 
	 * @access private
	 * @return bool
	 */
	private function validateExecution() {
		if ($this->app->isAdmin ()) {
			return false;	
		}
		
		// Get the dispatched option, view and id
		$option = $this->app->input->get('option');
		$view = $this->app->input->get('view');
		$id = $this->app->input->getInt('id');
		
		// Fallback for old extension using a task mapping
		if(!$view && $this->app->input->get('task')) {
			$view = $this->app->input->get('task');
		}
		
		// Retrieve any valid exclusions/inclusions filters by ID
		$exclusionsFilters = array();
		if(trim($this->params->get('exclusions_filters', null))) {
			$exclusionsFilters = explode(PHP_EOL, trim($this->params->get('exclusions_filters'), null));
			if(!empty($exclusionsFilters)) {
				foreach ($exclusionsFilters as &$exclusionFilter) {
					$exclusionFilter = trim($exclusionFilter);
				}
			}
		}
		if(trim($this->params->get('inclusions_filters', null))) {
			$inclusionsFilters = explode(PHP_EOL, trim($this->params->get('inclusions_filters'), null));
			if(!empty($inclusionsFilters)) {
				foreach ($inclusionsFilters as &$inclusionFilter) {
					$inclusionFilter = trim($inclusionFilter);
				}
			}
		}
		
		// Build the dispatched firm
		$fullDispatchedFirm = str_replace('com_', '', $option) . '.' . $view . '.' . $id;
		
		// Invalid request, an option and a view are required in order to process the AMP execution
		if(!$option || !$view) {
			return false;
		}
		
		// Additional exclusions filters validation for entities in the excluded containing category
		if($catid = $this->app->input->get('catid')) {
			if(isset($inclusionsFilters) && in_array(str_replace('com_', '', $option) . '.category.' . $catid , $inclusionsFilters)) {
				return true;
			}
			
			if(in_array(str_replace('com_', '', $option) . '.category.' . $catid , $exclusionsFilters)) {
				return false;
			}
		}
		
		// Concatenate the dispatched option view
		$dispatchedOptionView = $option . '.' . $view;
		
		// Get and parse the filter options
		$ampComponentsViews = $this->params->get('amp_components_views', array('com_content.article'));
		if(!is_array($ampComponentsViews)) {
			$ampComponentsViews = array('com_content.article');
		}
		
		// An invalid execution detected for this component and view, no inclusions are matched for the ID
		if(isset($inclusionsFilters) && in_array($dispatchedOptionView, $ampComponentsViews) && !in_array($fullDispatchedFirm, $inclusionsFilters)) {
			return false;
		}
		
		// A valid execution detected for this component and view, no exclusions are matched for the ID
		if(in_array($dispatchedOptionView, $ampComponentsViews) && !in_array($fullDispatchedFirm, $exclusionsFilters)) {
			return true;
		}
		
		/*
		 * Ensure that we do not have a AMP url with invalid execution, in such case redirect to the canonical
		 * JAmpHelper::$canonicalUrl
		 */
		$currentUri = JUri::current();
		$ampSuffix = $this->params->get ( 'amp_suffix', 'amp' );
		// Check if we have plugins exclusions activated
		if(preg_match('/\/' . $ampSuffix . '$|\.' . $ampSuffix . '\./i', $currentUri)) {
			if(!class_exists('JAmpHelper')) {
				require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/helper.php');
			}
			$this->app->redirect(trim(JAmpHelper::$canonicalUrl, '/'));
		}
		
		return false;
	}
	
	/**
	 * Check if the url is an absolute URL in some way
	 * 
	 * @param string $url
	 * @return bool
	 */
	private function isFullyQualified($url) {
		$isFullyQualified = substr($url, 0, 7) == 'http://' || substr($url, 0, 8) == 'https://' || substr($url, 0, 2) == '//';
		return $isFullyQualified;
	}
	
	/**
	 * Debug mode rendering
	 * 
	 * @access private
	 * @param String $ampLink
	 * @return void
	 */
	private function debugBar($ampLink) {
		$debugWidth = $this->params->get('debug_mode_width', 320);
		$debugHeight = $this->params->get('debug_mode_height', 480);
		
		// Purify the debug toolbar amp link
		$ampLink = trim($ampLink, '/');
		
		// If debug mode render a toolbar with a AMP link to test it
		$this->doc->addScriptDeclaration('jQuery(function($){$("body").append("<div id=\"jamp_debug_toolbar\">URL of the AMP version of this page: <a onclick=\"window.open(\'' . $ampLink . '\', \'jamp\', \'width=' . $debugWidth . ',height=' . $debugHeight . '\');return false;\" href=\"' . $ampLink . '\">' . $ampLink . '</a> | <a class=\"amptest\" target=\"_blank\" href=\"https://search.google.com/search-console/amp?url=' . rawurlencode($ampLink) . '\">AMP test</a></div>")});');
		$this->doc->addStyleDeclaration('#jamp_debug_toolbar a{color:#005e8d;}#jamp_debug_toolbar{position:fixed;z-index:999999;bottom:0;left:0;width:100%;background-color:#ff9800;padding:20px;font-size:20px;line-height:30px;}a.amptest{border:1px solid #333;border-radius:5px;background-color:#CCC;padding:4px 8px;}');
	}
	
	/**
	 * Store at this stage the plugin params into the JAmpHelper core class
	 *
	 * @access public
	 * @return boolean
	 */
	public function onAfterRoute() {
		// Inject objects to the static class helper
		if(! $this->startApp) {
			return false;
		}
		if(class_exists('JAmpHelper')) {
			JAmpHelper::$pluginParams = &$this->params;
		}
	}
	
	/**
	 * Adds to the document <head> of normal canonical pages, the rel ampthml metatag
	 * for the component.view that are enabled to have a valid AMP version
	 * 
	 * @access public
	 * @return void
	 */
	public function onAfterDispatch() {
		// Check if the app can start, if yes a valid AMP request is detected
		if (! $this->validateExecution()) {
			return;
		}
		
		// Never execute it on an AMP page
		if($this->startApp) {
			return;
		}
		
		// Retrieve Uri informations
		$path = $this->originalUri->getPath ();
		$query = $this->originalUri->getQuery ();
		$base = rtrim($this->originalUri->getScheme() . '://' . $this->originalUri->getHost(), '/') . '/';
		
		// Check for base URL domain Google AMP Cache activation
		if($this->params->get('google_amp_cache_activation', 0)) {
			// Is a https domain?
			if(stripos($base, 'https://') !== false) {
				$base = str_replace('https://', '', $base);
				$base = 'https://cdn.ampproject.org/c/s/' . $base;
			} else {
				$base = str_replace('http://', '', $base);
				$base = 'https://cdn.ampproject.org/c/' . $base;
			}
		}
		
		// Assignment and amp suffix detection
		$rawPath = $path;
		$ampSuffix = $this->params->get ( 'amp_suffix', 'amp' );
		
		// The Joomla SEF could be disabled, in such case we can avoid further processing
		if(!$this->app->get ( 'sef', 1)) {
			$rawAmpUrl = trim($base, '/') . '/' . trim($path, '/') . '?' . $query . '&' . $ampSuffix . '=1';
			// Add the amphtml rel metatag
			$this->doc->addHeadLink ( htmlspecialchars ( $rawAmpUrl, ENT_COMPAT, 'UTF-8' ), 'amphtml' );
			// Check if the debug mode is activated
			if($this->params->get('debug_mode', 0)) {
				$this->debugBar($rawAmpUrl);
			}
			return;
		}
		
		// Does it have a query string to remove/split?
		if (empty ( $query )) {
			$questionMarkPosition = strpos ( $path, '?' );
			if ($questionMarkPosition !== false) {
				$path = substr ( $path, 0, $questionMarkPosition );
				$query = substr ( $rawPath, $questionMarkPosition );
			}
		}
		
		// Does it start with a slash? (but not a protocol relative URL)
		$hasLeadingSlash = substr ( $path, 0, 1 ) == '/' && substr ( $path, 0, 2 ) != '//';
		
		// Does it end with a slash?
		$hasTrailingSlash = substr ( $path, - 1 ) == '/';
		
		// Do we have an html suffix?
		$htmlSuffix = null;
		$joomlaHtmlSuffix = $this->app->get ( 'sef_suffix', 1 );
		if ( $joomlaHtmlSuffix ) {
			$htmlSuffix = '.html';
		}
		if (defined ( 'SH404SEF_IS_RUNNING' )) {
			if(class_exists('Sh404sefFactory')) {
				$htmlSuffix = Sh404sefFactory::getConfig ()->suffix;
			}
		}
		
		if (substr ( $path, - 5 ) == $htmlSuffix) {
			$ampPath = substr ( $path, 0, - 5 ) . (empty ( $ampSuffix ) ? '' : '.' . $ampSuffix) . $htmlSuffix;
		} 		
		// Is a slash
		else if ($path == '/') {
			$ampPath = $ampSuffix;
		} 		
		// Ends with a slash
		else if ($hasTrailingSlash) {
			$ampPath = $path . (empty ( $ampSuffix ) ? '' : $ampSuffix . '/');
		} 		
		// Anything else
		else {
			$ampPath = $path . (empty ( $ampSuffix ) ? '' : '/' . $ampSuffix);
		}
		
		// Normalize path
		$ampPath = ltrim ( $ampPath, '/' );
		
		// Full URL, make sure the path was not already fully qualified
		if (! $this->isFullyQualified ( $ampPath )) {
			// apply index.php
			if (! $this->app->get ( 'sef_rewrite' ) && stripos($ampPath, 'index.php') === false) {
				$ampPath = 'index.php/' . $ampPath;
			}
			$ampPath = $base . $ampPath;
		}
		
		if(empty($query)) {
			$ampLink = $ampPath . $query;
		} else {
			$ampLink = trim($ampPath, '?') . '?' . trim($query, '?');
		}
		
		// Add the amphtml rel metatag
		$this->doc->addHeadLink ( htmlspecialchars ( rtrim($ampLink, '/'), ENT_COMPAT, 'UTF-8' ), 'amphtml' );
		
		// Check if the debug mode is activated
		if($this->params->get('debug_mode', 0)) {
			$this->debugBar($ampLink);
		}
	}
	
	/**
	 * Here happens the magic, the Joomla output is blocked and converted to AMP
	 *
	 * @access public
	 */
	public function onAfterRender() {
		// Check if the app can start, if yes a valid AMP request is detected
		if (! $this->validateExecution()) {
			return;
		}
		
		// Check if the AMP execution app is requested and activated
		if(! $this->startApp) {
			return false;
		}
		
		// Inject objects to the static class helper
		JAmpHelper::$document = &$this->doc;
		JAmpHelper::$application = &$this->app;
		JAmpHelper::$componentOutput = $this->doc->getBuffer ( 'component' );
		
		// Generate the AMP template
		require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/simplehtmldom.php');
		require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/template/main.php');
		
		// Exit the Joomla process here
		jexit ();
	}
	
	/**
	 * onAfterInitialise handler to manage content and system plugins exclusion
	 *
	 * @access	public
	 * @return void
	 */
	public function onAfterInitialise() {
		if ($this->app->isAdmin ()) {
			return;
		}
		
		// Register event handlers and store pristine URI parserule, before that the Joomla! router modifies it
		$joomlaRouter = $this->app->getRouter ();
		if(defined('JRouter::PROCESS_BEFORE')) {
			$joomlaRouter->attachParseRule ( array (
					$this,
					'preProcessParseRule'
			), JRouter::PROCESS_BEFORE );
		} else {
			$joomlaRouter->attachParseRule ( array (
					$this,
					'preProcessParseRule'
			));
		}
		
		// Detect in some way an AMP URL request at this stage before all routing will happen
		$ampSuffix = $this->params->get ( 'amp_suffix', 'amp' );
		$currentUri = JUri::current();
		
		// Check if we have plugins exclusions activated
		if($this->params->get('enable_plugins_exclusions', 0) && preg_match('/\/' . $ampSuffix . '$|\.' . $ampSuffix . '\./i', $currentUri)) {
			// Load plugins to exclude
			$excludedPlugins = $this->params->get('plugins_excluded', 0);
			if(is_array($excludedPlugins) && !in_array('0', $excludedPlugins)) {
				// Ensure preloading of needed plugins groups to be excluded
				JPluginHelper::importPlugin('content');
				JPluginHelper::importPlugin('system');
		
				// Instance of the subject JEventDispatcher
				$dispatcher = JEventDispatcher::getInstance();
		
				foreach ($excludedPlugins as $excludedPlugin) {
					list($pluginType, $pluginName) = explode('-', $excludedPlugin);
					// Build plugins structure
					$pluginToExclude = array('type'=>$pluginType, 'name'=>$pluginName);
		
					$pluginClassName = 'Plg' . ucfirst($pluginToExclude['type']) . ucfirst($pluginToExclude['name']);
					// Manage plugins exclusions at a early stage in the Joomla CMS app execution lifecycle
		
					if(class_exists($pluginClassName)) {
						// Get plugin observer object based on type/name
						$plugin = JPluginHelper::getPlugin($pluginToExclude['type'], $pluginToExclude['name']);
						// Instantiate the observer object and inject the subject for the attach
						$pluginInstanceToExclude = new $pluginClassName($dispatcher, (array) $plugin);
						// Now search and detach it
						$dispatcher->detach($pluginInstanceToExclude);
					}
				}
			}
		}
		
		// Always detach the Responsivizer plugin template switcher
		if(preg_match('/\/' . $ampSuffix . '$|\.' . $ampSuffix . '\./i', $currentUri)) {
			$pluginResponsivizerClassName = 'PlgSystemResponsivizer';
			if(class_exists($pluginResponsivizerClassName)) {
				// Instance of the subject JEventDispatcher
				$dispatcher = JEventDispatcher::getInstance();
				// Get plugin observer object based on type/name
				$plugin = JPluginHelper::getPlugin('system', 'responsivizer');
				// Instantiate the observer object and inject the subject for the attach
				$pluginResponsivizerInstanceToExclude = new $pluginResponsivizerClassName($dispatcher, (array) $plugin);
				// Now search and detach it
				$dispatcher->detach($pluginResponsivizerInstanceToExclude);
			}
		}
	}
	
	/**
	 * Validate the request before it's parsed and fully routed to identify the AMP suffix, 
	 * and alter the request so that it's parsed normally
	 * 
	 * We set a flag so that at a later stage we know that this is an AMP request
	 *
	 * @param
	 *        	$router
	 * @param
	 *        	$uri
	 */
	public function preProcessParseRule(&$router, &$uri) {
		// Store some immutable data
		$uriBase = $uri->base ();
		$uriBasePath = $uri->base ( true );
		$query = null;
		
		$originalPath = str_replace ( $uriBasePath, '', $this->originalUri->getPath () );
		
		$hasTrailingSlash = substr ( $originalPath, - 1 ) == '/';
		
		// Start the list of suffixes to be parsed and recognized as AMP pages
		$ampSuffix = $this->params->get ( 'amp_suffix', 'amp' );
		$ampSuffixes = array (
				array (
						'suffix' => $ampSuffix,
						'replacer' => '' 
				),
				array (
						'suffix' => $ampSuffix . '/',
						'replacer' => '/' 
				) 
		);
			
		// Do we have an html suffix?
		$htmlSuffix = null;
		$joomlaHtmlSuffix = $this->app->get ( 'sef_suffix', 1 );
		if ( $joomlaHtmlSuffix ) {
			$htmlSuffix = '.html';
		}
		if (defined ( 'SH404SEF_IS_RUNNING' )) {
			if (class_exists ( 'Sh404sefFactory' )) {
				$htmlSuffix = Sh404sefFactory::getConfig ()->suffix;
			}
		}
		// If the suffix is enabled and present add it to the list of suffixes
		if (! empty ( $htmlSuffix )) {
			$ampSuffixes [] = array (
					'suffix' => '.' . $ampSuffix . $htmlSuffix,
					'replacer' => $htmlSuffix 
			);
		}
		
		// Check if SEF URLs are on
		$sefUrlsActive = $this->app->get ( 'sef', 1);
		if(!$sefUrlsActive) {
			// Add a new suffix for no sef active
			$query = $uri->getQuery();
			$ampSuffixes [] = array (
					'suffix' => '&' . $ampSuffix . '=1',
					'replacer' => ''
			);
		}
		
		// look for one of the possible suffixes at the end of the incoming URL
		foreach ( $ampSuffixes as $suffixId => $suffix ) {
			$fullSuffix = substr ( $suffix ['suffix'], 0, 1 ) == '.' ? $suffix ['suffix'] : '/' . $suffix ['suffix'];
			if ($originalPath == $suffix ['suffix'] || 			// home page
			substr ( $originalPath, - strlen ( $fullSuffix ) ) == $fullSuffix || 
			substr ( $query, - strlen ( $suffix['suffix'] ) ) == $suffix['suffix']) {
				$currentSuffix = $fullSuffix;
				// NB: we use here $uri->getpath() instead of $originalPath
				// because $uri->getpath() has already been urldecoded by the router
				// and this is what we need to set back in the router for futher processing
				// however the JURI class incorrectly remove the trailing slash in incoming
				// URLs, so we have to compensate for that
				$currentPath = $uri->getPath () . ($hasTrailingSlash ? '/' : '');
				$newPath = substr ( $currentPath, 0, - strlen ( $fullSuffix ) ) . $suffix ['replacer'];
				
				// Set here the valid AMP request detected
				$this->startApp = true;
				break;
			}
		}
		
		if (defined ( 'SH404SEF_IS_RUNNING' ) && $this->startApp) {
			if (class_exists ( 'Sh404sefFactory' )) {
				$htmlSuffix = Sh404sefFactory::getConfig ()->suffix;
				if(!stripos($originalPath, $htmlSuffix)) {
					$nonSefUrl = '';
					$testPath = $newPath;
					$urlRecord = ShlMvcModel_Base::getInstance('Sefurls', 'Sh404sefModel')->getNonSefUrlRecordFromDatabase($testPath, $nonSefUrl);
					$urlType = $urlRecord['status'];
					// Not found sh404sef URL, invert trailing slash and newpath
					if($urlType == -2) {
						$hasTrailingSlash = !$hasTrailingSlash;
						$newPath = $hasTrailingSlash ? $newPath . '/' : trim($newPath, '/');
					}
				}
			}
		}
		
		// Valid AMP request detected
		if ($this->startApp && $sefUrlsActive) {
			// Suppresses trailing slash as Joomla does
			$truncatedPath = $hasTrailingSlash ? ltrim ( $newPath, '/' ) : trim ( $newPath, '/' );
			
			$uri->setPath ( $truncatedPath );
			$uriQuery = $uri->getQuery ();
			$uriQuery = empty ( $uriQuery ) ? '' : '?' . $uriQuery;
			$urlRewritePrefix = 'index.php';
			if (defined ( 'SH404SEF_IS_RUNNING' ) && empty ( $truncatedPath )) {
				// sh404SEF: on home page, we don't keep the index.php bit as Joomla does
				$urlRewritePrefix = '';
			}
			
			$truncatedPath = rtrim ( $truncatedPath, '/' );
			
			if (! $this->app->get ( 'sef_rewrite' )) {
				// not using URL Rewriting, stick index.php at beginning of new URL
				$truncatedPath = $urlRewritePrefix . (empty ( $truncatedPath ) ? '' : '/' . $truncatedPath);
			}
			
			// find the canonical based on the previously calculated bits
			// not forgetting to also re-append the trailing slash Joomla stripped
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/image.php');
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/fastimage.php');
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/helper.php');
			JAmpHelper::$canonicalUrl = $uriBase . $truncatedPath . ($hasTrailingSlash ? '/' : '') . $uriQuery;
			JAmpHelper::$ampUrl = JUri::current ();
			JAmpHelper::$isJAmpRequest = true;
			
			// Home page URL
			$sefLanguage = null;
			if ($sefUrlsActive && $this->isMultilangEnabled()) {
				$sefs = JLanguageHelper::getLanguages('sef');
				$path = $uri->getPath();
				$parts = explode('/', $path);
				if(array_key_exists($parts[0], $sefs)) {
					$sefLanguage = $parts[0] . '/';
				}
			}
			JAmpHelper::$ampHomeUrl = JUri::base(false) . $sefLanguage . $ampSuffix;
		} elseif($this->startApp && !$sefUrlsActive) {
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/image.php');
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/fastimage.php');
			require_once (JPATH_ROOT . '/plugins/system/jamp/core/includes/helper.php');
			$path = $uri->getPath ();
			$base = $uri->base();
			$rawCanonicalUrl = trim($base, '/') . '/' . trim($path, '/') . '?' . str_replace('&' . $ampSuffix . '=1', '', $query);
			$rawAmpUrl = trim($base, '/') . '/' . trim($path, '/') . '?' . $query;
				
			JAmpHelper::$canonicalUrl = $rawCanonicalUrl;
			JAmpHelper::$ampUrl = $rawAmpUrl;
			JAmpHelper::$isJAmpRequest = true;
		}
		
		// Return a void array of empty parsed vars not needed
		return array();
	}
	
	/** Manage the Joomla updater based on the user license
	 *
	 * @access public
	 * @return void
	 */
	public function onInstallerBeforePackageDownload(&$url, &$headers) {
		$uri 	= JUri::getInstance($url);
		$parts 	= explode('/', $uri->getPath());
		$app = JFactory::getApplication();
		if ($uri->getHost() == 'storejextensions.org' && in_array('plg_jamp.zip', $parts)) {
			// Init as false unless the license is valid
			$validUpdate = false;
	
			// Manage partial language translations
			$jLang = JFactory::getLanguage();
			$jLang->load('plg_system_jamp', JPATH_ADMINISTRATOR, 'en-GB', true, true);
	
			// Email license validation API call and &$url building construction override
			if(!class_exists('JPluginHelper')) {
				jimport('joomla.plugin.helper');
			}
			$plugin = JPluginHelper::getPlugin('system', 'jamp');
			$pluginParams = json_decode($plugin->params);
			$registrationEmail = $pluginParams->registration_email;
	
			// License
			if($registrationEmail) {
				$prodCode = 'jamp';
				$cdFuncUsed = 'str_' . 'ro' . 't' . '13';
	
				// Retrieve license informations from the remote REST API
				$apiResponse = null;
				$apiEndpoint = $cdFuncUsed('uggc' . '://' . 'fgberwrkgrafvbaf' . '.bet') . "/option,com_easycommerce/action,licenseCode/email,$registrationEmail/productcode,$prodCode";
				if (function_exists('curl_init')){
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $apiEndpoint);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					$apiResponse = curl_exec($ch);
					curl_close($ch);
				}
				$objectApiResponse = json_decode($apiResponse);
	
				if(!is_object($objectApiResponse)) {
					// Message user about error retrieving license informations
					$app->enqueueMessage(JText::_('PLG_JAMP_ERROR_RETRIEVING_LICENSE_INFO'));
				} else {
					if(!$objectApiResponse->success) {
						switch ($objectApiResponse->reason) {
							// Message user about the reason the license is not valid
							case 'nomatchingcode':
								$app->enqueueMessage(JText::_('PLG_JAMP_LICENSE_NOMATCHING'));
								break;
	
							case 'expired':
								// Message user about license expired on $objectApiResponse->expireon
								$app->enqueueMessage(JText::sprintf('PLG_JAMP_LICENSE_EXPIRED', $objectApiResponse->expireon));
								break;
						}
							
					}
	
					// Valid license found, builds the URL update link and message user about the license expiration validity
					if($objectApiResponse->success) {
						$url = $cdFuncUsed('uggc' . '://' . 'fgberwrkgrafvbaf' . '.bet' . '/WNZC1306TSPQcfqe3243779923sernj35td1pync7756b.ugzy');
	
						$validUpdate = true;
						$app->enqueueMessage(JText::sprintf('PLG_JAMP_EXTENSION_UPDATED_SUCCESS', $objectApiResponse->expireon));
					}
				}
			} else {
				// Message user about missing email license code
				$app->enqueueMessage(JText::_('PLG_JAMP_MISSING_REGISTRATION_EMAIL_ADDRESS'));
			}
	
			if(!$validUpdate) {
				$app->enqueueMessage(JText::_('PLG_JAMP_UPDATER_STANDARD_ADVISE'), 'notice');
			}
		}
	}
	
	function onContentPrepareForm($form, $data) {
		// Only works on JForms
		if (!($form instanceof JForm)) return false;
	
		// which belong to the following components
		$components_list = array(
				"com_content.article",
				"com_categories.categorycom_content"
		);
	
		$formName = $form->getName();
		if (!in_array($formName, $components_list)) return true;
	
		switch ($form->getName()) {
			case 'com_content.article':
				$form->load('<form>
								<fields name="metadata">
									<fieldset name="jmetadata">
									    <field name="jamp_image" type="media" default="" label="AMP image" description="Choose and AMP meta image" />
									</fieldset>
								</fields>
							</form>');
				break;
				
			case 'com_categories.categorycom_content':
				$form->load('<form>
								<fields name="metadata">
									<fieldset name="jmetadata">
									    <field name="jamp_schema_type" class="btn-group" type="radio" default="" label="AMP metadata type" description="Choose a meta data type for your contents">
											<option value="">Use global</option>
											<option value="NewsArticle">NewsArticle</option>
											<option value="BlogPosting">BlogPosting</option>
										</field>
									</fieldset>
								</fields>
							</form>');
				break;
		}
		
		
		return true;
	}
	
	/**
	 * Plugin constructor
	 *
	 * @access public
	 */
	public function __construct(&$subject, $config = array()) {
		$this->app = JFactory::getApplication ();

		// Validate side execution
		if ($this->app->isAdmin ()) {
			// Allow plugin execution validation for com_content.article.edit, com_categories.category.edit and com_installer.update
			if( $this->app->input->get('option') == 'com_content' &&
				$this->app->input->get('view') == 'article' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_content' &&
				$this->app->input->get('task') == 'article.apply' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_content' &&
				$this->app->input->get('task') == 'article.save' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_content' &&
				$this->app->input->get('task') == 'article.save2new' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_content' &&
				$this->app->input->get('task') == 'article.save2copy' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_categories' &&
				$this->app->input->get('extension') == 'com_content' &&
				$this->app->input->get('view') == 'category' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_categories' &&
				$this->app->input->get('extension') == 'com_content' &&
				$this->app->input->get('task') == 'category.apply' ||
				$this->app->input->get('option') == 'com_categories' &&
				$this->app->input->get('extension') == 'com_content' &&
				$this->app->input->get('task') == 'category.save' ||
				$this->app->input->get('option') == 'com_categories' &&
				$this->app->input->get('extension') == 'com_content' &&
				$this->app->input->get('task') == 'category.save2new' ||
				$this->app->input->get('option') == 'com_categories' &&
				$this->app->input->get('extension') == 'com_content' &&
				$this->app->input->get('task') == 'category.save2copy' &&
				$this->app->input->get('layout') == 'edit' ||
				$this->app->input->get('option') == 'com_installer' &&
				$this->app->input->get('view') == 'update' &&
				$this->app->input->get('task') == 'update.update'
			) {
				parent::__construct ( $subject, $config );
				return true;
			} else {
				return false;
			}
		}
		
		// Validating URL to avoid to instantiate a JDocumentHTML in this phase BEFORE component routing, it would break any other formats with a forced JDocumentHTML
		$defaultExclusions = array (
				'xml',
				'json',
				'raw',
				'feed',
				'jmap',
				'sitemap',
				'rss' 
		);

		$uriObject = JUri::getInstance ();
		$path = explode ( '/', $uriObject->getPath () );
		$query = $uriObject->getQuery ( true );
		if (@array_intersect ( $defaultExclusions, $path ) || @array_intersect ( $defaultExclusions, $query )) {
			return false;
		}
		
		// Validate doc type execution
		$this->doc = JFactory::getDocument ();
		if ($this->doc->getType () !== 'html' || $this->app->input->getCmd ( 'tmpl' ) == 'component') {
			return false;
		}
		JFactory::$document = null;
		
		// Ensure compatibility for at least Joomla 3.2+
		if(version_compare(JVERSION, '3.2', '<')) {
			return false;
		}
		
		$this->originalUri = JUri::getInstance ();
		
		parent::__construct ( $subject, $config );
		
		// Ensure to kill the page caching on Joomla 3.7+
		if(version_compare(JVERSION, '3.7', '>=') && JPluginHelper::isEnabled('system', 'cache')) {
			$ampSuffix = $this->params->get ( 'amp_suffix', 'amp' );
			$currentUri = JUri::current();
			if(strpos($currentUri, '.' . $ampSuffix) || strpos($currentUri, '/' . $ampSuffix)) {
				JFactory::getUser()->set('guest', null);
			}
		}
	}
}